use crate::{decks::new_blackjack_shoe, hand::Hand};

pub struct BlackjackGame {
    shoe: Hand,
    player_hand: Hand,
    dealer_hand: Hand,
    state: GameState,
}

pub enum PlayResult {
    DealerBlackJack, // Dealer has a blackjack. Player does not
    PlayerBlackJack, // Player has a blackjack. Dealer does not
    PushBlackjack,   // Player and dealer have a blackjack
    Push,            // Dealer has a Blackjack and so does the Player
    Bust,            // Player bust
    DealerBust,      // Dealer bust
    StandPlayerLose, // Dealer stands and wins
    StandPlayerWin,  // Dealer stands and loses
    Continue,        // Game goes on as normal
    InvalidMove,
}

#[derive(Clone, Copy,Debug)]
pub enum PlayMoves {
    Hit,
    Stand,
    DoubleDown,
    Bet, // At the Start of the game
}

pub enum GameState {
    Empty,      // No cards have been dealt
    Over,       // Game is over
    PlayerTurn, // Its the turn of the player
}

impl BlackjackGame {
    pub fn new() -> Self {
        BlackjackGame {
            shoe: new_blackjack_shoe(6),
            player_hand: Hand::new(),
            dealer_hand: Hand::new(),
            state: GameState::Empty,
        }
    }

    pub fn get_dealer_hand(&self) -> &Hand {
        &self.dealer_hand
    }

    pub fn get_player_hand(&self) -> &Hand {
        &self.player_hand
    }

    pub fn get_state(&self) -> &GameState {
        &self.state
    }

    pub fn play(&mut self, action: PlayMoves) -> PlayResult {
        match (&self.state, action) {
            (GameState::Empty, PlayMoves::Bet) => {
                self.player_hand.add_card(self.shoe.pop_card().unwrap());
                self.dealer_hand.add_card(self.shoe.pop_card().unwrap());

                self.player_hand.add_card(self.shoe.pop_card().unwrap());
                self.dealer_hand.add_card(self.shoe.pop_card().unwrap());

                match (self.dealer_hand.is_backjack(),self.player_hand.is_backjack()) {
                    (true,true) => {
                        self.state = GameState::Over;
                        PlayResult::PushBlackjack
                    },
                    (true,false) => {
                        self.state = GameState::Over;
                        PlayResult::DealerBlackJack
                    },
                    (false,true) => {
                        self.state = GameState::Over;
                        PlayResult::PlayerBlackJack
                    },
                    (false,false) => {
                        self.state = GameState::PlayerTurn;
                        PlayResult::Continue
                    },
                }
            }
            (GameState::PlayerTurn, PlayMoves::Hit) => {
                self.player_hand.add_card(self.shoe.pop_card().unwrap());

                return match self.player_hand.get_blackjack_value().cmp(&21) {
                    std::cmp::Ordering::Equal => {
                        self.state = GameState::Over;
                        self.dealer_play()
                    }
                    std::cmp::Ordering::Greater => {
                        self.state = GameState::Over;
                        PlayResult::Bust
                    }
                    std::cmp::Ordering::Less => PlayResult::Continue,
                };
            },
            (GameState::PlayerTurn, PlayMoves::DoubleDown) => {
                if self.player_hand.count() > 2{
                    return PlayResult::InvalidMove;
                }
                
                self.player_hand.add_card(self.shoe.pop_card().unwrap());
                self.state = GameState::Over;
                return self.dealer_play();
            },
            (GameState::PlayerTurn, PlayMoves::Stand) => {
                self.state = GameState::Over;
                return self.dealer_play();
            }
            (GameState::Over, _)
            | (GameState::Empty, _)
            | (GameState::PlayerTurn, PlayMoves::Bet) => {
                return PlayResult::InvalidMove;
            }
        }
    }

    fn dealer_play(&mut self) -> PlayResult {
        // Stand on 17 or above
        // Hit on 16 and below

        loop {
            let dealer_value = self.dealer_hand.get_blackjack_value();
            if dealer_value >= 17 {
                let player_value = self.player_hand.get_blackjack_value();

                return match dealer_value.cmp(&player_value) {
                    std::cmp::Ordering::Equal => PlayResult::Push,
                    std::cmp::Ordering::Greater => PlayResult::StandPlayerLose,
                    std::cmp::Ordering::Less => PlayResult::StandPlayerWin,
                };
            }

            self.dealer_hand.add_card(self.shoe.pop_card().unwrap());

            if self.dealer_hand.get_blackjack_value() > 21 {
                return PlayResult::DealerBust;
            }
        }
    }
}
